<?php

// composer require guzzlehttp/guzzle league/uri league/uri-components
require __DIR__ . '/vendor/autoload.php';

use GuzzleHttp\Client;
use League\Uri\Uri;
use League\Uri\UriModifier;
use OSS\OssClient;
use OSS\Core\OssException;
use RingCentral\Psr7\Response;

// 获取存储桶名称
$bucket = getenv('oss_bucket');
// OSS 全局存储
$oss = null;
// 支持的域名
$domains = [
    'github.com',
    'codeload.github.com',
    'github-production-release-asset-2e65be.s3.amazonaws.com',
];

function initializer($context) {
    global $oss;

    if(is_null($oss)) {
        // 实例化 OSS
        // $oss = new OssClient(
        //     'yourAccessKeyId',
        //     'yourAccessKeySecret',
        //     'https://oss-' . $context['region'] . '.aliyuncs.com'
        // );
        $oss = new OssClient(
            $context['credentials']['accessKeyId'],
            $context['credentials']['accessKeySecret'],
            'https://oss-' . $context['region'] . '-internal.aliyuncs.com',
            false,
            $context['credentials']['securityToken']
        );
    }
}

function handler($request, $context): Response{
    global $oss, $bucket, $domains;

    // 获取请求的 Body
    $body = json_decode($request->getBody()->getContents(), true);

    if(!isset($body['url'])) {
        return error('参数错误');
    }

    $host = parse_url($body['url'], PHP_URL_HOST);

    // 判断 url 参数中的域名是否在支持名单之中
    if(!in_array($host, $domains)) {
        return error('不支持此域名');
    }

    $unique = get_unique($body['url']);

    if(!$unique) {
        return error('获取标识失败');
    }

    $oss_list = $oss->listObjects($bucket, [
                        'delimiter' => '/',
                        'prefix'    => $unique . '/',
                        'max-keys'  => 1,
                        'marker'    => '',
                    ])
                    ->getObjectList();

    if($oss_list) {
        return result([
            'sign' => get_sign($oss_list[0]->getKey()),
        ]);
    }

    // 生成临时文件路径
    $tmp_filepath = '/tmp/' . $unique;

    $handle = fopen($tmp_filepath, 'w');

    $client = new Client([
        'base_uri'      => '',
        'verify'        => true,
        'sink'          => $tmp_filepath,
        'curl.options'  => [
            'CURLOPT_RETURNTRANSFER'    => true,
            'CURLOPT_FILE'              => $handle,
        ],
    ]);

    $response = $client->get($body['url']);

    if($response->getStatusCode() != 200) {
        return error('读取文件失败');
    }

    // OSS 保存文件名
    $oss_filename = get_filename($response->getHeaderLine('content-disposition'));

    if($oss_filename === false) {
        return error('不支持此链接');
    }

    // OSS 保存路径
    $oss_filepath = sprintf('%s/%s', $unique, $oss_filename);

    try{
        // 上传到 OSS
        $oss->uploadFile($bucket, $oss_filepath, $tmp_filepath);
    } catch(OssException $e) {
        return error('OSS: ' . $e->getMessage());
    }

    // 删除临时文件
    unlink($tmp_filepath);

    return result([
        'sign' => get_sign($oss_filepath),
    ]);
}

/**
 * 获取签名
 *
 * @param string  $path  路径
 * @return string
 */
function get_sign(string $path) {
    global $oss, $bucket;

    $cdn_domain = getenv('cdn_domain');

    // 切换成 CDN 域名
    if($cdn_domain) {
        return sprintf('https://%s/%s', $cdn_domain, $path);
    }

    // 获取临时签名
    $sign = (string) $oss->signUrl($bucket, $path, 60 * 60);

    $has_accelerate = (bool) getenv('has_accelerate');

    // 切换成传输加速域名
    if($has_accelerate) {
        return str_replace(parse_url($sign, PHP_URL_HOST), $bucket . '.oss-accelerate.aliyuncs.com', $sign);
    }

    // 需要防止使用内网域名
    return str_replace('-internal', '', $sign);
}

/**
 * 获取标识
 * 链接的域名是 github.com 且是正常下载链接应该返回 301 跳转地址的
 * codeload.github.com 下的资源需要先避免缓存然后对头部的 ETag 进行 md5 生成
 * 其他需根据链接的 path 段进行 md5 生成
 *
 * @param string  $url  链接
 * @return string|bool
 */
function get_unique(string $url) {
    $host = parse_url($url, PHP_URL_HOST);

    if(strtolower($host) == 'github.com') {
        $header = get_headers($url, 1);

        if(!isset($header['location'])) {
            return false;
        }

        $url = $header['location'];
    }

    if(parse_url($url, PHP_URL_HOST) == 'codeload.github.com') {
        $url = UriModifier::appendQuery(Uri::createFromString($url), 'timestamp=' . time());
        return md5(get_headers($url, 1)['ETag']);
    }

    return is_null($host) ? false : md5(parse_url($url, PHP_URL_PATH));
}

/**
 * 获取文件名
 * 根据请求头部的 Content-Disposition 截取出 filename 的值
 * wiki: https://developer.mozilla.org/zh-CN/docs/Web/HTTP/Headers/Content-Disposition
 *
 * @param string  $disposition  Content-Disposition
 * @return string
 */
function get_filename($disposition) {
    if(is_array($disposition)) {
        $disposition = $disposition[0];
    }

    $disposition = explode('filename=', $disposition);

    return count($disposition) === 2 ? $disposition[1] : false;
}

/**
 * 返回成功
 *
 * @param array  $data  数据
 * @return RingCentral\Psr7\Response
 */
function result($data = []): Response {
    return new Response(
        200,
        [
            'Content-type' => 'application/json',
        ],
        json_encode([
            'code'  => 1,
            'data'  => $data,
        ])
    );
}

/**
 * 返回错误
 *
 * @param string  $msg  错误信息
 * @return RingCentral\Psr7\Response
 */
function error($msg): Response {
    return new Response(
        200,
        [
            'Content-type' => 'application/json',
        ],
        json_encode([
            'code'  => 0,
            'msg'   => $msg,
        ])
    );
}

?>